package org.seamframework.extension;

import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.AbstractSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.enterprise.inject.spi.Annotated;

public abstract class Reannotated implements Annotated {
	
	protected abstract Annotated getDelegate();
	
	private Map<Class<? extends Annotation>, Annotation> annotations = 
			new HashMap<Class<? extends Annotation>, Annotation>();
	
	public <X extends Annotation> void redefine(Class<X> annotationType, AnnotationRedefinition<X> visitor) {
		if ( isAnnotationPresent(annotationType) ) {
			annotations.put(annotationType, visitor.redefine(getAnnotation(annotationType), this));
		}
	}
	
	public void define(Annotation ann) {
		annotations.put(ann.annotationType(), ann);
	}
	
	public boolean isAnnotationWithMetatypePresent(Class<? extends Annotation> metaannotationType) {
		for (Annotation ann: getAnnotations()) {
			if ( ann.annotationType().isAnnotationPresent(metaannotationType) ) {
				return true;
			}
		}
		return false;
	}
	
	public Set<Annotation> getAnnotationsWithMetatype(Class<? extends Annotation> metaannotationType) {
		Set<Annotation> set = new HashSet<Annotation>();
		for (Annotation ann: getAnnotations()) {
			if ( ann.annotationType().isAnnotationPresent(metaannotationType) ) {
				set.add(ann);
			}
		}
		return set;
	}
	
	public abstract Class<?> getJavaClass();

	private class AnnotationSet extends AbstractSet<Annotation> {
		
		List<Annotation> list = new ArrayList<Annotation>();
		
		AnnotationSet() {
			list.addAll( annotations.values() );
			for (Annotation ann: getDelegate().getAnnotations()) {
				if ( !annotations.containsKey(ann.annotationType()) ) {
					list.add(ann);
				}
			}
		}

		@Override
		public Iterator<Annotation> iterator() {
			return list.iterator();
		}

		@Override
		public int size() {
			return list.size();
		}
		
	}
	
	
	@Override
	@SuppressWarnings("unchecked")
	public <T extends Annotation> T getAnnotation(Class<T> annotationType) {
		Annotation ann = annotations.get(annotationType);
		if (ann!=null) { 
			return (T) ann;
		}
		else {
			return getDelegate().getAnnotation(annotationType);
		}
	}

	@Override
	public Set<Annotation> getAnnotations() {
		return new AnnotationSet();
	}

	@Override
	public Type getBaseType() {
		return getDelegate().getBaseType();
	}

	@Override
	public Set<Type> getTypeClosure() {
		return getDelegate().getTypeClosure();
	}

	@Override
	public boolean isAnnotationPresent(Class<? extends Annotation> annotationType) {
		return annotations.containsKey(annotationType) || 
				getDelegate().isAnnotationPresent(annotationType);
	}

}
